"""
LOLZTEAM it's library that contains all the methods of the Lolzteam API (Market/Forum/Antipublic)

You can find full documentation here -> https://github.com/AS7RIDENIED/LOLZTEAM
"""

from .API import Forum
from .API import Market
from .API import Antipublic

from . import Exceptions
from . import Tweaks
from . import Constants
from .BBCODE import BBCODE
