class AS7RID_FUCK_UP(Exception):
    pass


class INVALID_PROXY_TYPE(Exception):
    pass


class USERID_REQUIRED_FOR_BATCH(Exception):
    pass


class INVALID_ARG(Exception):
    pass


class UNEXPECTED_ARG(Exception):
    pass


class URL_IS_DIFFERENT_FROM_BASE_MARKET(Exception):
    pass


class MISSING_SCOPE(Exception):
    pass


class BAD_TOKEN(Exception):
    pass

class PATH_NOT_DEFINED(Exception):
    pass